"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.GetRenderMapVisitor = void 0;
const prettier_1 = require("prettier");
const nodes = __importStar(require("../../nodes"));
const shared_1 = require("../../shared");
const logs_1 = require("../../shared/logs");
const visitors_1 = require("../../visitors");
const RenderMap_1 = require("../RenderMap");
const utils_1 = require("../utils");
const fragments_1 = require("./fragments");
const GetTypeManifestVisitor_1 = require("./GetTypeManifestVisitor");
const ImportMap_1 = require("./ImportMap");
const nameTransformers_1 = require("./nameTransformers");
const DEFAULT_PRETTIER_OPTIONS = {
    semi: true,
    singleQuote: true,
    trailingComma: 'es5',
    useTabs: false,
    tabWidth: 2,
    arrowParens: 'always',
    printWidth: 80,
    parser: 'typescript',
};
class GetRenderMapVisitor extends visitors_1.BaseThrowVisitor {
    constructor(options = {}) {
        var _a, _b, _c, _d, _e, _f, _g;
        super();
        this.program = null;
        const nameTransformers = {
            ...nameTransformers_1.DEFAULT_NAME_TRANSFORMERS,
            ...options.nameTransformers,
        };
        this.nameApi = (0, nameTransformers_1.getNameApi)(nameTransformers);
        this.options = {
            renderParentInstructions: (_a = options.renderParentInstructions) !== null && _a !== void 0 ? _a : false,
            formatCode: (_b = options.formatCode) !== null && _b !== void 0 ? _b : true,
            prettierOptions: {
                ...DEFAULT_PRETTIER_OPTIONS,
                ...options.prettierOptions,
            },
            dependencyMap: (_c = options.dependencyMap) !== null && _c !== void 0 ? _c : {},
            typeManifestVisitor: (_d = options.typeManifestVisitor) !== null && _d !== void 0 ? _d : new GetTypeManifestVisitor_1.GetTypeManifestVisitor(this.nameApi),
            byteSizeVisitor: (_e = options.byteSizeVisitor) !== null && _e !== void 0 ? _e : new visitors_1.GetByteSizeVisitor(),
            resolvedInstructionInputVisitor: (_f = options.resolvedInstructionInputVisitor) !== null && _f !== void 0 ? _f : new visitors_1.GetResolvedInstructionInputsVisitor(),
            asyncResolvers: (_g = options.asyncResolvers) !== null && _g !== void 0 ? _g : [],
            nameTransformers,
        };
    }
    visitRoot(root) {
        var _a, _b;
        (_b = (_a = this.byteSizeVisitor).registerDefinedTypes) === null || _b === void 0 ? void 0 : _b.call(_a, nodes.getAllDefinedTypes(root));
        const programsToExport = root.programs.filter((p) => !p.internal);
        const programsWithErrorsToExport = programsToExport.filter((p) => p.errors.length > 0);
        const accountsToExport = nodes
            .getAllAccounts(root)
            .filter((a) => !a.internal);
        const instructionsToExport = nodes
            .getAllInstructionsWithSubs(root, !this.options.renderParentInstructions)
            .filter((i) => !i.internal);
        const definedTypesToExport = nodes
            .getAllDefinedTypes(root)
            .filter((t) => !t.internal);
        const hasAnythingToExport = programsToExport.length > 0 ||
            accountsToExport.length > 0 ||
            instructionsToExport.length > 0 ||
            definedTypesToExport.length > 0;
        const ctx = {
            root,
            programsToExport,
            programsWithErrorsToExport,
            accountsToExport,
            instructionsToExport,
            definedTypesToExport,
            hasAnythingToExport,
        };
        const map = new RenderMap_1.RenderMap();
        if (hasAnythingToExport) {
            map.add('shared/index.ts', this.render('sharedPage.njk', ctx));
        }
        if (programsToExport.length > 0) {
            map.add('programs/index.ts', this.render('programsIndex.njk', ctx));
        }
        if (programsWithErrorsToExport.length > 0) {
            map.add('errors/index.ts', this.render('errorsIndex.njk', ctx));
        }
        if (accountsToExport.length > 0) {
            map.add('accounts/index.ts', this.render('accountsIndex.njk', ctx));
        }
        if (instructionsToExport.length > 0) {
            map.add('instructions/index.ts', this.render('instructionsIndex.njk', ctx));
        }
        if (definedTypesToExport.length > 0) {
            map.add('types/index.ts', this.render('definedTypesIndex.njk', ctx));
        }
        return map
            .add('index.ts', this.render('rootIndex.njk', ctx))
            .add('global.d.ts', this.render('globalTypesPage.njk', ctx))
            .mergeWith(...root.programs.map((program) => (0, visitors_1.visit)(program, this)));
    }
    visitProgram(program) {
        this.program = program;
        const renderMap = new RenderMap_1.RenderMap()
            .mergeWith(...program.accounts.map((account) => (0, visitors_1.visit)(account, this)))
            .mergeWith(...program.definedTypes.map((type) => (0, visitors_1.visit)(type, this)));
        // Internal programs are support programs that
        // were added to fill missing types or accounts.
        // They don't need to render anything else.
        if (program.internal) {
            this.program = null;
            return renderMap;
        }
        if (program.errors.length > 0) {
            const programErrorsFragment = (0, fragments_1.getProgramErrorsFragment)({
                programNode: program,
                nameApi: this.nameApi,
            });
            renderMap.add(`errors/${(0, shared_1.camelCase)(program.name)}.ts`, this.render('errorsPage.njk', {
                imports: new ImportMap_1.ImportMap()
                    .mergeWith(programErrorsFragment)
                    .toString(this.options.dependencyMap),
                programErrorsFragment,
            }));
        }
        const programFragment = (0, fragments_1.getProgramFragment)({
            programNode: program,
            nameApi: this.nameApi,
        });
        renderMap.add(`programs/${(0, shared_1.camelCase)(program.name)}.ts`, this.render('programsPage.njk', {
            imports: new ImportMap_1.ImportMap()
                .mergeWith(programFragment)
                .toString(this.options.dependencyMap),
            programFragment,
        }));
        renderMap.mergeWith(...nodes
            .getAllInstructionsWithSubs(program, !this.options.renderParentInstructions)
            .map((ix) => (0, visitors_1.visit)(ix, this)));
        this.program = null;
        return renderMap;
    }
    visitAccount(account) {
        if (!this.program) {
            throw new Error('Account must be visited inside a program.');
        }
        const scope = {
            accountNode: account,
            programNode: this.program,
            typeManifest: (0, visitors_1.visit)(account, this.typeManifestVisitor),
            typeManifestVisitor: this.typeManifestVisitor,
            nameApi: this.nameApi,
        };
        const accountTypeFragment = (0, fragments_1.getAccountTypeFragment)(scope);
        const accountFetchHelpersFragment = (0, fragments_1.getAccountFetchHelpersFragment)(scope);
        const accountSizeHelpersFragment = (0, fragments_1.getAccountSizeHelpersFragment)(scope);
        const accountPdaHelpersFragment = (0, fragments_1.getAccountPdaHelpersFragment)(scope);
        const imports = new ImportMap_1.ImportMap().mergeWith(accountTypeFragment, accountFetchHelpersFragment, accountSizeHelpersFragment, accountPdaHelpersFragment);
        return new RenderMap_1.RenderMap().add(`accounts/${(0, shared_1.camelCase)(account.name)}.ts`, this.render('accountsPage.njk', {
            imports: imports.toString(this.options.dependencyMap),
            accountTypeFragment,
            accountFetchHelpersFragment,
            accountSizeHelpersFragment,
            accountPdaHelpersFragment,
        }));
    }
    visitInstruction(instruction) {
        if (!this.program) {
            throw new Error('Instruction must be visited inside a program.');
        }
        const scope = {
            instructionNode: instruction,
            programNode: this.program,
            renamedArgs: this.getRenamedArgsMap(instruction),
            dataArgsManifest: (0, visitors_1.visit)(instruction.dataArgs, this.typeManifestVisitor),
            extraArgsManifest: (0, visitors_1.visit)(instruction.extraArgs, this.typeManifestVisitor),
            resolvedInputs: (0, visitors_1.visit)(instruction, this.resolvedInstructionInputVisitor),
            asyncResolvers: this.options.asyncResolvers,
            nameApi: this.nameApi,
        };
        // Fragments.
        const instructionTypeFragment = (0, fragments_1.getInstructionTypeFragment)({
            ...scope,
            withSigners: false,
        });
        const instructionTypeWithSignersFragment = (0, fragments_1.getInstructionTypeFragment)({
            ...scope,
            withSigners: true,
        });
        const instructionDataFragment = (0, fragments_1.getInstructionDataFragment)(scope);
        const instructionExtraArgsFragment = (0, fragments_1.getInstructionExtraArgsFragment)(scope);
        const instructionFunctionHighLevelAsyncFragment = (0, fragments_1.getInstructionFunctionHighLevelFragment)({ ...scope, useAsync: true });
        const instructionFunctionHighLevelSyncFragment = (0, fragments_1.getInstructionFunctionHighLevelFragment)({ ...scope, useAsync: false });
        const instructionFunctionLowLevelFragment = (0, fragments_1.getInstructionFunctionLowLevelFragment)(scope);
        const instructionParseFunctionFragment = (0, fragments_1.getInstructionParseFunctionFragment)(scope);
        // Imports and interfaces.
        const imports = new ImportMap_1.ImportMap().mergeWith(instructionTypeFragment, instructionTypeWithSignersFragment, instructionDataFragment, instructionExtraArgsFragment, instructionFunctionHighLevelAsyncFragment, instructionFunctionHighLevelSyncFragment, instructionFunctionLowLevelFragment, instructionParseFunctionFragment);
        return new RenderMap_1.RenderMap().add(`instructions/${(0, shared_1.camelCase)(instruction.name)}.ts`, this.render('instructionsPage.njk', {
            instruction,
            imports: imports.toString(this.options.dependencyMap),
            instructionTypeFragment,
            instructionTypeWithSignersFragment,
            instructionDataFragment,
            instructionExtraArgsFragment,
            instructionFunctionHighLevelAsyncFragment,
            instructionFunctionHighLevelSyncFragment,
            instructionFunctionLowLevelFragment,
            instructionParseFunctionFragment,
        }));
    }
    visitDefinedType(definedType) {
        const scope = {
            typeNode: definedType.data,
            name: definedType.name,
            manifest: (0, visitors_1.visit)(definedType, this.typeManifestVisitor),
            typeDocs: definedType.docs,
            encoderDocs: [],
            decoderDocs: [],
            codecDocs: [],
            nameApi: this.nameApi,
        };
        const typeWithCodecFragment = (0, fragments_1.getTypeWithCodecFragment)(scope);
        const typeDataEnumHelpersFragment = (0, fragments_1.getTypeDataEnumHelpersFragment)(scope);
        const imports = new ImportMap_1.ImportMap()
            .mergeWith(typeWithCodecFragment, typeDataEnumHelpersFragment)
            .remove('generatedTypes', [
            this.nameApi.dataType(definedType.name),
            this.nameApi.dataArgsType(definedType.name),
            this.nameApi.encoderFunction(definedType.name),
            this.nameApi.decoderFunction(definedType.name),
            this.nameApi.codecFunction(definedType.name),
        ]);
        return new RenderMap_1.RenderMap().add(`types/${(0, shared_1.camelCase)(definedType.name)}.ts`, this.render('definedTypesPage.njk', {
            imports: imports.toString({
                ...this.options.dependencyMap,
                generatedTypes: '.',
            }),
            typeWithCodecFragment,
            typeDataEnumHelpersFragment,
        }));
    }
    get typeManifestVisitor() {
        return this.options.typeManifestVisitor;
    }
    get byteSizeVisitor() {
        return this.options.byteSizeVisitor;
    }
    get resolvedInstructionInputVisitor() {
        return this.options.resolvedInstructionInputVisitor;
    }
    getRenamedArgsMap(instruction) {
        const argNames = [
            ...instruction.dataArgs.struct.fields.map((field) => field.name),
            ...instruction.extraArgs.struct.fields.map((field) => field.name),
        ];
        const duplicateArgs = argNames.filter((e, i, a) => a.indexOf(e) !== i);
        if (duplicateArgs.length > 0) {
            throw new Error(`Duplicate args found: [${duplicateArgs.join(', ')}] in instruction [${instruction.name}].`);
        }
        const allNames = [
            ...instruction.accounts.map((account) => account.name),
            ...argNames,
        ];
        const duplicates = allNames.filter((e, i, a) => a.indexOf(e) !== i);
        if (duplicates.length === 0)
            return new Map();
        (0, logs_1.logWarn)(`[JavaScriptExperimental] Accounts and args of instruction [${instruction.name}] have the following ` +
            `conflicting attributes [${duplicates.join(', ')}]. ` +
            `Thus, the arguments have been renamed to avoid conflicts in the input type.`);
        return new Map(duplicates.map((name) => [(0, shared_1.mainCase)(name), (0, shared_1.mainCase)(`${name}Arg`)]));
    }
    render(template, context, options) {
        const code = (0, utils_1.resolveTemplate)(`${__dirname}/templates`, template, context, options);
        return this.options.formatCode
            ? (0, prettier_1.format)(code, this.options.prettierOptions)
            : code;
    }
}
exports.GetRenderMapVisitor = GetRenderMapVisitor;
//# sourceMappingURL=GetRenderMapVisitor.js.map