"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getInstructionParseFunctionFragment = void 0;
const common_1 = require("./common");
function getInstructionParseFunctionFragment(scope) {
    const { instructionNode, programNode, dataArgsManifest, nameApi } = scope;
    const hasAccounts = instructionNode.accounts.length > 0;
    const hasOptionalAccounts = instructionNode.accounts.some((account) => account.isOptional);
    const minimumNumberOfAccounts = instructionNode.optionalAccountStrategy === 'omitted'
        ? instructionNode.accounts.filter((account) => !account.isOptional).length
        : instructionNode.accounts.length;
    const hasData = !!instructionNode.dataArgs.link ||
        instructionNode.dataArgs.struct.fields.length > 0;
    if (!hasAccounts && !hasData) {
        return (0, common_1.fragment)('');
    }
    const dataTypeFragment = (0, common_1.fragment)(instructionNode.dataArgs.link
        ? dataArgsManifest.strictType.render
        : nameApi.dataType(instructionNode.dataArgs.name));
    const decoderFunction = instructionNode.dataArgs.link
        ? dataArgsManifest.decoder.render
        : `${nameApi.decoderFunction(instructionNode.dataArgs.name)}()`;
    if (instructionNode.dataArgs.link) {
        dataTypeFragment.mergeImportsWith(dataArgsManifest.strictType, dataArgsManifest.decoder);
    }
    return (0, common_1.fragmentFromTemplate)('instructionParseFunction.njk', {
        instruction: instructionNode,
        programAddress: programNode.publicKey,
        instructionParsedType: nameApi.instructionParsedType(instructionNode.name),
        instructionParseFunction: nameApi.instructionParseFunction(instructionNode.name),
        dataTypeFragment,
        decoderFunction,
        hasAccounts,
        hasOptionalAccounts,
        minimumNumberOfAccounts,
        hasData,
    })
        .mergeImportsWith(dataTypeFragment)
        .addImports('solanaInstructions', ['IInstruction'])
        .addImports('solanaInstructions', hasAccounts ? ['IInstructionWithAccounts', 'IAccountMeta'] : [])
        .addImports('solanaInstructions', hasData ? ['IInstructionWithData'] : []);
}
exports.getInstructionParseFunctionFragment = getInstructionParseFunctionFragment;
//# sourceMappingURL=instructionParseFunction.js.map