"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DEFAULT_NAME_TRANSFORMERS = exports.getNameApi = void 0;
const shared_1 = require("../../shared");
function getNameApi(transformers) {
    const helpers = {
        capitalize: shared_1.capitalize,
        titleCase: shared_1.titleCase,
        pascalCase: shared_1.pascalCase,
        camelCase: shared_1.camelCase,
        kebabCase: shared_1.kebabCase,
        snakeCase: shared_1.snakeCase,
    };
    return Object.fromEntries(Object.entries(transformers).map(([key, transformer]) => [
        key,
        (name) => transformer(name, helpers),
    ]));
}
exports.getNameApi = getNameApi;
exports.DEFAULT_NAME_TRANSFORMERS = {
    dataType: (name) => `${(0, shared_1.pascalCase)(name)}`,
    dataArgsType: (name) => `${(0, shared_1.pascalCase)(name)}Args`,
    encoderFunction: (name) => `get${(0, shared_1.pascalCase)(name)}Encoder`,
    decoderFunction: (name) => `get${(0, shared_1.pascalCase)(name)}Decoder`,
    codecFunction: (name) => `get${(0, shared_1.pascalCase)(name)}Codec`,
    accountType: (name) => `${(0, shared_1.pascalCase)(name)}`,
    accountSeedsType: (name) => `${(0, shared_1.pascalCase)(name)}Seeds`,
    // accountData: (name) => `${pascalCase(name)}AccountData`,
    accountDecodeFunction: (name) => `decode${(0, shared_1.pascalCase)(name)}`,
    accountFetchFunction: (name) => `fetch${(0, shared_1.pascalCase)(name)}`,
    accountFetchAllFunction: (name) => `fetchAll${(0, shared_1.pascalCase)(name)}`,
    accountSafeFetchFunction: (name) => `safeFetch${(0, shared_1.pascalCase)(name)}`,
    accountSafeFetchAllFunction: (name) => `safeFetchAll${(0, shared_1.pascalCase)(name)}`,
    accountFindPdaFunction: (name) => `find${(0, shared_1.pascalCase)(name)}Pda`,
    accountFetchFromSeedsFunction: (name) => `fetch${(0, shared_1.pascalCase)(name)}FromSeeds`,
    accountSafeFetchFromSeedsFunction: (name) => `safeFetch${(0, shared_1.pascalCase)(name)}FromSeeds`,
    accountGetSizeFunction: (name) => `get${(0, shared_1.pascalCase)(name)}Size`,
    dataEnumFunction: (name) => `${(0, shared_1.camelCase)(name)}`,
    isDataEnumFunction: (name) => `is${(0, shared_1.pascalCase)(name)}`,
    instructionAsyncInputType: (name) => `${(0, shared_1.pascalCase)(name)}AsyncInput`,
    instructionAsyncInputWithSignersType: (name) => `${(0, shared_1.pascalCase)(name)}AsyncInputWithSigners`,
    instructionSyncInputType: (name) => `${(0, shared_1.pascalCase)(name)}Input`,
    instructionSyncInputWithSignersType: (name) => `${(0, shared_1.pascalCase)(name)}InputWithSigners`,
    instructionType: (name) => `${(0, shared_1.pascalCase)(name)}Instruction`,
    instructionWithSignersType: (name) => `${(0, shared_1.pascalCase)(name)}InstructionWithSigners`,
    // instructionData: (name) => `${pascalCase(name)}InstructionData`,
    instructionAsyncFunction: (name) => `get${(0, shared_1.pascalCase)(name)}InstructionAsync`,
    instructionSyncFunction: (name) => `get${(0, shared_1.pascalCase)(name)}Instruction`,
    instructionRawFunction: (name) => `get${(0, shared_1.pascalCase)(name)}InstructionRaw`,
    instructionParsedType: (name) => `Parsed${(0, shared_1.pascalCase)(name)}Instruction`,
    instructionParseFunction: (name) => `parse${(0, shared_1.pascalCase)(name)}Instruction`,
    programType: (name) => `${(0, shared_1.pascalCase)(name)}Program`,
    programAddressConstant: (name) => `${(0, shared_1.snakeCase)(name).toUpperCase()}_PROGRAM_ADDRESS`,
    programCreateFunction: (name) => `create${(0, shared_1.pascalCase)(name)}Program`,
    programErrorClass: (name) => `${(0, shared_1.pascalCase)(name)}ProgramError`,
    programErrorCodeEnum: (name) => `${(0, shared_1.pascalCase)(name)}ProgramErrorCode`,
    programErrorCodeMap: (name) => `${(0, shared_1.camelCase)(name)}ProgramErrorCodeMap`,
    programGetErrorFromCodeFunction: (name) => `get${(0, shared_1.pascalCase)(name)}ProgramErrorFromCode`,
    resolverFunction: (name) => `${(0, shared_1.camelCase)(name)}`,
};
//# sourceMappingURL=nameTransformers.js.map