import type { ConfigureOptions } from 'nunjucks';
import { Options as PrettierOptions } from 'prettier';
import * as nodes from '../../nodes';
import { ImportFrom } from '../../shared';
import { BaseThrowVisitor, ResolvedInstructionInput, Visitor } from '../../visitors';
import { RenderMap } from '../RenderMap';
import { TypeManifest } from './TypeManifest';
import { NameTransformers } from './nameTransformers';
export type GetRenderMapOptions = {
    renderParentInstructions?: boolean;
    formatCode?: boolean;
    prettierOptions?: PrettierOptions;
    dependencyMap?: Record<ImportFrom, string>;
    typeManifestVisitor?: Visitor<TypeManifest>;
    byteSizeVisitor?: Visitor<number | null> & {
        registerDefinedTypes?: (definedTypes: nodes.DefinedTypeNode[]) => void;
    };
    resolvedInstructionInputVisitor?: Visitor<ResolvedInstructionInput[]>;
    asyncResolvers?: string[];
    nameTransformers?: Partial<NameTransformers>;
};
export declare class GetRenderMapVisitor extends BaseThrowVisitor<RenderMap> {
    readonly options: Required<GetRenderMapOptions> & {
        nameTransformers: NameTransformers;
    };
    private program;
    private nameApi;
    constructor(options?: GetRenderMapOptions);
    visitRoot(root: nodes.RootNode): RenderMap;
    visitProgram(program: nodes.ProgramNode): RenderMap;
    visitAccount(account: nodes.AccountNode): RenderMap;
    visitInstruction(instruction: nodes.InstructionNode): RenderMap;
    visitDefinedType(definedType: nodes.DefinedTypeNode): RenderMap;
    get typeManifestVisitor(): Visitor<TypeManifest>;
    get byteSizeVisitor(): Visitor<number | null> & {
        registerDefinedTypes?: ((definedTypes: nodes.DefinedTypeNode[]) => void) | undefined;
    };
    get resolvedInstructionInputVisitor(): Visitor<ResolvedInstructionInput[]>;
    protected getRenamedArgsMap(instruction: nodes.InstructionNode): Map<string, string>;
    protected render(template: string, context?: object, options?: ConfigureOptions): string;
}
